@@ -477,7 +477,18 @@ class BaseOptimizer(ABC):
         self.library = library
 
         self.training_params, self.int_params = _get_params(self.library, device)
-        self.training_params = _update_params(self.training_params, override_params)
+        self.training_params = self._update_params(self.training_params, override_params)
+
+    def _update_params(self, hyperopt_params, params):
+        # Replace values in dict
+        if params is not None:
+            for k, v in params.items():
+                if k in hyperopt_params:
+                    if v is None:
+                        del hyperopt_params[k]
+                    else:
+                        hyperopt_params[k] = v
+        return hyperopt_params
 
     def optimize(self,
                  X_train,
@@ -526,18 +537,18 @@ class BaseOptimizer(ABC):
                           trials=self.trials)
 
         # Extract the best params
-        best_params, best_n = self._get_best_params(hyperopt_f, self.trials)
+        best_params, best_n = self._get_best_params(hyperopt_f)
 
         # Builds a dict containing the best params and the best n iterations
-        best = {'params': best_params, 'n': best_n}
+        self.best = {'params': best_params, 'n': best_n}
 
         # Save results to json file
         if savepath:
-            self._save_json(best, savepath)
+            self.save_json(self.best, savepath)
 
-        return best
+        return self.best
 
-    def _get_best_params(self, best, trials):
+    def _get_best_params(self, best):
         """
         """
         best_params = best.copy()
@@ -547,7 +558,7 @@ class BaseOptimizer(ABC):
         for key in self.training_params:
             if key not in best_params.keys():
                 best_params[key] = self.training_params[key]
-        best_n_iterations = trials.best_trial['result']['best_n_iters'] if 'best_n_iters' in trials.best_trial['result'] else None
+        best_n_iterations = self.trials.best_trial['result']['best_n_iters'] if 'best_n_iters' in self.trials.best_trial['result'] else None
         return best_params, best_n_iterations
 
     def plot(self, best='min', invert=False, figsize=(15, 6)):
@@ -573,7 +584,7 @@ class BaseOptimizer(ABC):
                     len(losses), best_iteration, best_value))
         plt.show()
 
-    def _save_json(self, data, path, verbose=1):
+    def save_json(self, data, path, verbose=1):
         if path[path.rfind('.'):] != '.json':
             path = path + '.json'
         with open(path, 'w') as fp:
